# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-07-02 08:15
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("aimmo", "0014_add_worksheet_model"),
    ]

    operations = [
        migrations.AddField(
            model_name="game",
            name="worksheet",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="aimmo.Worksheet",
            ),
        ),
    ]



import logging
from typing import TYPE_CHECKING, Dict
from dataclasses import dataclass, field

LOGGER = logging.getLogger(__name__)

if TYPE_CHECKING:
    from socketio import AsyncServer


@dataclass
class CollectedTurnActions:
    turn_number: int
    player_id_to_actions: Dict[int, object] = field(default_factory=dict)

    def get_action_for_player(self, player_id: int) -> object:
        try:
            return self.player_id_to_actions[player_id]
        except KeyError:
            return {"action_type": "wait"}


class TurnCollector:
    def __init__(self, socketio_server: "AsyncServer"):
        self.socketio_server = socketio_server
        self.register_action_received_event()

    def new_turn(self, turn_number):
        self.collected_turns = CollectedTurnActions(turn_number)

    def register_action_received_event(self):
        @self.socketio_server.on("action")
        async def action_received(sid, data):
            session_data = await self.socketio_server.get_session(sid)
            if data["turnCount"] == self.collected_turns.turn_number:
                player_id = session_data["id"]
                self.collected_turns.player_id_to_actions[player_id] = data["action"]

        return action_received



from aimmo.models import Game
from django import template

from common.permissions import logged_in_as_student, logged_in_as_teacher

register = template.Library()


@register.inclusion_tag("players/dropdown.html", takes_context=True)
def game_dropdown_list(context, base_url):
    return get_user_playable_games(context, base_url)


def get_user_playable_games(context, base_url):
    user = context.request.user
    if logged_in_as_student(user):
        playable_games = user.userprofile.student.class_field.game
    elif logged_in_as_teacher(user):
        playable_games = Game.objects.filter(
            game_class__teacher=user.userprofile.teacher, is_archived=False
        )
    else:
        playable_games = Game.objects.none()
    return {
        "base_url": base_url,
        "open_play_games": playable_games if user.is_authenticated else None,
    }



import errno
import os
import platform
import stat
import subprocess
import sys
from subprocess import CalledProcessError

try:
    from urllib.request import urlretrieve, urlopen
except ImportError:
    from urllib import urlretrieve, urlopen

BASE_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
TEST_BIN = os.path.join(BASE_DIR, "test-bin")
OS = platform.system().lower()
FILE_SUFFIX = ".exe" if OS == "windows" else ""
KUBECTL = os.path.join(TEST_BIN, "kubectl%s" % FILE_SUFFIX)
MINIKUBE = os.path.join(TEST_BIN, "minikube%s" % FILE_SUFFIX)
FNULL = open(os.devnull, "w")


def log(message):
    sys.stderr.write(message + "\n")


def run_command(args, capture_output=False):
    try:
        if capture_output:
            return subprocess.check_output(args)
        else:
            subprocess.check_call(args)
    except CalledProcessError as e:
        log("Command failed with exit status %d: %s" % (e.returncode, " ".join(args)))
        raise


def run_command_async(args, capture_output=False):
    env = os.environ.copy()
    if capture_output is True:
        p = subprocess.Popen(args, stdout=FNULL, stderr=subprocess.STDOUT, env=env)
    else:
        p = subprocess.Popen(args, env=env)
    return p


def binary_exists(filename):
    # Check if binary is callable on our path
    try:
        run_command([filename], True)
        return True
    except OSError:
        return False


def download_exec(url, dest):
    dest = urlretrieve(url, dest)[0]
    make_exec(dest)


def make_exec(file):
    current_stat = os.stat(file)
    os.chmod(file, current_stat.st_mode | stat.S_IEXEC)


def get_latest_github_version(repo):
    result = urlopen("https://github.com/%s/releases/latest" % repo)
    return result.geturl().split("/")[-1]



from functools import wraps

from django.conf import settings
from django.utils.module_loading import import_string
from django.contrib.auth.models import User
from rest_framework import permissions

#: URL function for locating the game server, takes one parameter `game`
GAME_SERVER_URL_FUNCTION = getattr(settings, "AIMMO_GAME_SERVER_URL_FUNCTION", None)
GAME_SERVER_PORT_FUNCTION = getattr(settings, "AIMMO_GAME_SERVER_PORT_FUNCTION", None)
GAME_SERVER_SSL_FLAG = getattr(settings, "AIMMO_GAME_SERVER_SSL_FLAG", False)

MAX_LEVEL = 1



from os import urandom

from base64 import urlsafe_b64encode
from common.models import Class
from django.contrib.auth.models import User
from django.db import models
from django.utils import timezone

from aimmo import app_settings
from aimmo.worksheets import WORKSHEETS

DEFAULT_WORKSHEET_ID = 1

GAME_GENERATORS = [("Main", "Open World")] + [  # Default
    ("Level%s" % i, "Level %s" % i) for i in range(1, app_settings.MAX_LEVEL + 1)
]


def generate_auth_token():
    return urlsafe_b64encode(urandom(16))


class Game(models.Model):
    RUNNING = "r"
    STOPPED = "s"
    PAUSED = "p"
    STATUS_CHOICES = ((RUNNING, "running"), (STOPPED, "stopped"), (PAUSED, "paused"))

    name = models.CharField(max_length=100, blank=True, null=True)
    auth_token = models.CharField(max_length=48, blank=True)
    owner = models.ForeignKey(
        User,
        blank=True,
        null=True,
        related_name="owned_games",
        on_delete=models.SET_NULL,
    )
    game_class = models.OneToOneField(
        Class,
        verbose_name="Class",
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )
    public = models.BooleanField(default=False)
    can_play = models.ManyToManyField(
        User,
        related_name="playable_games",
        help_text="List of auth_user IDs of users who are allowed to play and have access to the game.",
    )
    completed = models.BooleanField(default=False)
    main_user = models.ForeignKey(
        User,
        blank=True,
        null=True,
        related_name="games_for_user",
        on_delete=models.SET_NULL,
    )
    static_data = models.TextField(blank=True, null=True)

    # Game config
    generator = models.CharField(
        max_length=20, choices=GAME_GENERATORS, default=GAME_GENERATORS[0][0]
    )
    target_num_cells_per_avatar = models.FloatField(default=16)
    target_num_score_locations_per_avatar = models.FloatField(default=0.5)
    score_despawn_chance = models.FloatField(default=0.05)
    target_num_pickups_per_avatar = models.FloatField(default=1.2)
    pickup_spawn_chance = models.FloatField(default=0.1)
    obstacle_ratio = models.FloatField(default=0.1)
    start_height = models.IntegerField(default=31)
    start_width = models.IntegerField(default=31)
    status = models.CharField(max_length=1, choices=STATUS_CHOICES, default=RUNNING)
    worksheet_id = models.IntegerField(default=DEFAULT_WORKSHEET_ID)
    is_archived = models.BooleanField(default=False)
    creation_time = models.DateTimeField(default=timezone.now, null=True)

    @property
    def is_active(self):
        return not self.completed

    @property
    def worksheet(self):
        return WORKSHEETS.get(self.worksheet_id)

    def __str__(self):
        return str(self.id)

    def can_user_play(self, user: User) -> bool:
        """Checks whether the given user has permission to play the game.

        A user can play the game if they are part of the game's class or
        the teacher of that class.

        Args:
            user: A standard django User object

        Returns:
            bool: True if user can play the game, False otherwise
        """
        try:
            return (
                self.game_class.students.filter(new_user=user).exists()
                or user == self.game_class.teacher.new_user
            )
        except AttributeError:
            return False

    def settings_as_dict(self):
        return {
            "GENERATOR": self.generator,
            "OBSTACLE_RATIO": self.obstacle_ratio,
            "PICKUP_SPAWN_CHANCE": self.pickup_spawn_chance,
            "SCORE_DESPAWN_CHANCE": self.score_despawn_chance,
            "START_HEIGHT": self.start_height,
            "START_WIDTH": self.start_width,
            "TARGET_NUM_CELLS_PER_AVATAR": self.target_num_cells_per_avatar,
            "TARGET_NUM_PICKUPS_PER_AVATAR": self.target_num_pickups_per_avatar,
            "TARGET_NUM_SCORE_LOCATIONS_PER_AVATAR": self.target_num_score_locations_per_avatar,
        }

    def save(self, *args, **kwargs):
        super(Game, self).full_clean()
        super(Game, self).save(*args, **kwargs)


class Avatar(models.Model):
    owner = models.ForeignKey(User, on_delete=models.CASCADE)
    game = models.ForeignKey(Game, on_delete=models.CASCADE)
    code = models.TextField()
    auth_token = models.CharField(max_length=24, default=generate_auth_token)

    class Meta:
        unique_together = ("owner", "game")



import asyncio
import logging
from typing import TYPE_CHECKING

from metrics import GAME_TURN_TIME
from simulation.avatar.avatar_manager import AvatarManager
from simulation.django_communicator import (
    DjangoCommunicator,
    GameMetadataFetchFailedError,
)
from simulation.simulation_runner import ConcurrentSimulationRunner

if TYPE_CHECKING:
    from turn_collector import TurnCollector
    from simulation.game_state import GameState

LOGGER = logging.getLogger(__name__)

TURN_TIME = 2


class GameRunner:
    def __init__(
        self,
        game_state_generator,
        communicator: DjangoCommunicator,
        port,
        turn_collector: "TurnCollector",
    ):
        super(GameRunner, self).__init__()

        self.game_state: "GameState" = game_state_generator(AvatarManager())
        self.communicator = communicator
        self.simulation_runner = ConcurrentSimulationRunner(
            communicator=self.communicator, game_state=self.game_state
        )
        self.turn_collector = turn_collector
        self.turn_collector.new_turn(self.game_state.turn_count)
        self._end_turn_callback = lambda: None

    def set_end_turn_callback(self, callback_method):
        self._end_turn_callback = callback_method

    def get_users_to_add(self, game_metadata):
        def player_is_new(_player):
            return (
                _player["id"]
                not in self.simulation_runner.game_state.avatar_manager.avatars_by_id.keys()
            )

        return [
            player["id"] for player in game_metadata["users"] if player_is_new(player)
        ]

    def get_users_to_delete(self, game_metadata):
        def player_in_avatar_manager_but_not_metadata(pid):
            return pid not in [player["id"] for player in game_metadata["users"]]

        return [
            player_id
            for player_id in self.simulation_runner.game_state.avatar_manager.avatars_by_id.keys()
            if player_in_avatar_manager_but_not_metadata(player_id)
        ]

    async def update_avatars(self):
        try:
            game_metadata = await self.communicator.get_game_metadata()
            users_to_add = self.get_users_to_add(game_metadata)
            users_to_delete = self.get_users_to_delete(game_metadata)

            self.simulation_runner.add_avatars(users_to_add)
            self.simulation_runner.delete_avatars(users_to_delete)
        except GameMetadataFetchFailedError:
            LOGGER.error("Game metadata fetch failed, not updating avatars this turn")
            pass

    async def update_simulation(self, player_id_to_serialized_actions):
        await self.simulation_runner.run_single_turn(player_id_to_serialized_actions)
        await self._end_turn_callback()
        self.game_state.turn_count += 1

    async def update(self):
        with GAME_TURN_TIME():
            await self.update_avatars()
            await self.update_simulation(self.turn_collector.collected_turns)
            self.game_state.avatar_manager.clear_all_avatar_logs()
            self.turn_collector.new_turn(self.game_state.turn_count)

    def _get_task_result_or_stop_loop(self, task):
        try:
            task.result()
        except Exception as e:
            LOGGER.exception(f"Unexpected error, stopping game loop: {e}")
            loop = asyncio.get_event_loop()
            loop.stop()

    async def run(self):
        while True:
            LOGGER.info(f"Starting turn {self.game_state.turn_count}")
            turn = asyncio.ensure_future(self.update())

            turn.add_done_callback(self._get_task_result_or_stop_loop)

            await asyncio.sleep(TURN_TIME)
            await turn



# Generated by Django 2.2.25 on 2022-01-14 15:50

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('aimmo', '0026_remove_worksheets'),
    ]

    operations = [
        migrations.AddField(
            model_name='game',
            name='creation_time',
            field=models.DateTimeField(default=django.utils.timezone.now, null=True),
        ),
        migrations.AddField(
            model_name='game',
            name='is_archived',
            field=models.BooleanField(default=False),
        ),
    ]



import math
import random
from abc import ABCMeta, abstractmethod
from collections import Counter, namedtuple
from logging import getLogger
from typing import List, Type

from simulation.cell import Cell
from simulation.interactables.interactable import _Interactable
from simulation.interactables.score_location import ScoreLocation
from simulation.location import Location

LOGGER = getLogger(__name__)

MapContext = namedtuple("MapContext", "num_avatars")


class _MapUpdater:
    __metaclass__ = ABCMeta

    @abstractmethod
    def update(self, world_map, context):
        raise NotImplementedError


class ScoreLocationUpdater(_MapUpdater):
    def update(self, world_map, context):
        for cell in world_map.score_cells():
            if random.random() < world_map.settings["SCORE_DESPAWN_CHANCE"]:
                cell.interactable.delete()

        new_num_score_locations = len(list(world_map.score_cells()))
        target_num_score_locations = int(
            math.ceil(
                context.num_avatars
                * world_map.settings["TARGET_NUM_SCORE_LOCATIONS_PER_AVATAR"]
            )
        )
        num_score_locations_to_add = (
            target_num_score_locations - new_num_score_locations
        )
        locations = world_map._spawn_location_finder.get_random_spawn_locations(
            num_score_locations_to_add
        )
        for cell in locations:
            cell.interactable = ScoreLocation(cell)


class PickupUpdater(_MapUpdater):
    """
    Generates artefacts based on the TARGET_NUM_PICKUPS_PER_AVATAR setting.
    """

    def __init__(self, pickup_types: List[Type[_Interactable]]) -> None:
        super().__init__()
        self.pickup_types = pickup_types

    def update(self, world_map, context):
        target_total_num_pickups = (
            context.num_avatars * world_map.settings["TARGET_NUM_PICKUPS_PER_AVATAR"]
        )
        target_num_pickups_per_type = int(
            math.ceil(target_total_num_pickups / len(self.pickup_types))
        )
        current_num_pickups = Counter(
            type(cell.interactable) for cell in world_map.pickup_cells()
        )
        for pickup_type in self.pickup_types:
            max_num_pickups_to_add = (
                target_num_pickups_per_type - current_num_pickups[pickup_type]
            )
            locations = world_map._spawn_location_finder.get_random_spawn_locations(
                max_num_pickups_to_add
            )
            for cell in locations:
                cell.interactable = pickup_type(cell)
                LOGGER.info(
                    "Adding new pickup at %s of type %s", cell, cell.interactable
                )



# Generated by Django 2.2.17 on 2021-01-19 17:23

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("aimmo", "0023_remove_level_attempt"),
    ]

    operations = [
        migrations.RunSQL(
            """
            CREATE TEMPORARY TABLE temp.aimmo_games_to_delete AS
            SELECT
                id
            FROM
                aimmo_game ag
            JOIN (
                SELECT
                    owner_id,
                    game_class_id,
                    MAX(id) target_game_id
                FROM
                    aimmo_game
                GROUP BY
                    owner_id,
                    game_class_id
                HAVING
                    count(*) > 1) duplicate_game ON
                ag.owner_id = duplicate_game.owner_id
                AND ag.game_class_id = duplicate_game.game_class_id
                AND ag.id != duplicate_game.target_game_id;

            DELETE
            FROM
                aimmo_avatar
            WHERE
                game_id IN (
                SELECT
                    id
                FROM
                    temp.aimmo_games_to_delete);

            DELETE
            FROM
                aimmo_game
            WHERE
                id IN (
                SELECT
                    id
                FROM
                    temp.aimmo_games_to_delete);

            DROP TABLE temp.aimmo_games_to_delete;
            """,
            migrations.RunSQL.noop,
        ),
        migrations.AlterField(
            model_name="game",
            name="game_class",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="common.Class",
                verbose_name="Class",
            ),
        ),
        migrations.AlterField(
            model_name="game",
            name="worksheet",
            field=models.ForeignKey(
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                to="aimmo.Worksheet",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="game",
            unique_together=set(),
        ),
    ]



from prometheus_client import Histogram
import os

# DO NOT use a metric more than once, it will probably break them. If you need to measure
# something in a similar or the same way as an existing metric, create a new one.

# Measures the time taken for the game to go through an entire turn (game runner's update method)
CUSTOM_BUCKET = [x / 10 for x in range(10, 61)]
GAME_TURN = Histogram(
    "game_turn_time",
    "Measures the time taken for the game to complete a single turn in seconds",
    buckets=CUSTOM_BUCKET,
)


def GAME_TURN_TIME():
    """
    Use for measuring the time it games for the game to complete a turn.

    This is measured using the Histogram datatype.
    """
    return GAME_TURN.time()



from __future__ import absolute_import

import logging
import os
import sys
import time

import django
from django.conf import settings

from .shell_api import log, run_command, run_command_async

ROOT_DIR_LOCATION = os.path.abspath(os.path.dirname((os.path.dirname(__file__))))

_MANAGE_PY = os.path.join(ROOT_DIR_LOCATION, "example_project", "manage.py")
_FRONTEND_BUNDLER_JS = os.path.join(
    ROOT_DIR_LOCATION, "game_frontend", "djangoBundler.js"
)

PROCESSES = []


def create_superuser_if_missing(username, password):
    from django.contrib.auth.models import User

    try:
        User.objects.get_by_natural_key(username)
    except User.DoesNotExist:
        log("Creating superuser %s with password %s" % (username, password))
        User.objects.create_superuser(
            username=username, email="admin@admin.com", password=password
        )


def build_worker_package():
    run_command(
        [os.path.join(ROOT_DIR_LOCATION, "aimmo_runner", "build_worker_wheel.sh")],
        capture_output=True,
    )


def build_frontend(using_cypress, capture_output):
    if using_cypress:
        run_command(["node", _FRONTEND_BUNDLER_JS], capture_output=capture_output)
    else:
        frontend_bundler = run_command_async(
            ["node", _FRONTEND_BUNDLER_JS], capture_output=capture_output
        )
        PROCESSES.append(frontend_bundler)


def start_game_servers(build_target, server_args):
    parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    sys.path.append(os.path.join(parent_dir, "aimmo_runner"))
    os.chdir(ROOT_DIR_LOCATION)

    # Import minikube here, so we can install the dependencies first
    from aimmo_runner import minikube

    minikube.start(build_target=build_target)

    server_args.append("0.0.0.0:8000")
    os.environ["AIMMO_MODE"] = "minikube"


def run(
    server_wait=True,
    using_cypress=False,
    capture_output=False,
    test_env=False,
    build_target=None,
):
    logging.basicConfig()

    build_worker_package()

    if test_env:
        os.environ.setdefault("DJANGO_SETTINGS_MODULE", "test_settings")
    else:
        sys.path.insert(0, os.path.join(ROOT_DIR_LOCATION, "example_project"))
        os.environ.setdefault("DJANGO_SETTINGS_MODULE", "example_project.settings")

    django.setup()

    if using_cypress:
        settings.DEBUG = False
        os.environ["LOAD_KUBE_CONFIG"] = "0"

    os.environ["NODE_ENV"] = "development" if settings.DEBUG else "production"

    build_frontend(using_cypress, capture_output)

    run_command(
        ["pip", "install", "-e", ROOT_DIR_LOCATION], capture_output=capture_output
    )

    if not test_env:
        run_command(
            ["python", _MANAGE_PY, "migrate", "--noinput"],
            capture_output=capture_output,
        )
        run_command(
            ["python", _MANAGE_PY, "collectstatic", "--noinput"],
            capture_output=capture_output,
        )

    create_superuser_if_missing(username="admin", password="admin")

    server_args = []
    if not using_cypress:
        start_game_servers(build_target, server_args)

    os.environ["SERVER_ENV"] = "local"
    server = run_command_async(
        ["python", _MANAGE_PY, "runserver"] + server_args,
        capture_output=capture_output,
    )
    PROCESSES.append(server)

    if server_wait:
        try:
            game.wait()
        except NameError:
            pass

        server.wait()

    return PROCESSES



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

setup(
    name="aimmo",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "eventlet",
        "requests",
        "djangorestframework==3.12.2",
    ],
    tests_require=[],
    test_suite="test_utils.test_suite.DjangoAutoTestSuite",
    zip_safe=False,
)



from logging import getLogger
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from simulation.game_state import GameState

LOGGER = getLogger(__name__)


class EffectApplier:
    """
    Applier for all interactables on the WorldMap.

    Handles the checking if conditions are met, then applies any effects for interactables
    where this occurs.
    """

    def apply(self, game_state: "GameState"):
        """Apply interactable effects to any avatar that is on a pickup cell."""
        for cell in game_state.world_map.interactable_cells():
            if cell.interactable.conditions_met(game_state):
                cell.interactable.apply()



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

setup(
    name="aimmo_runner",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "django==2.2.26",
        "django-autoconfig >= 0.3.6, < 1.0.0",
        "django-forms-bootstrap",
        "django-js-reverse",
        "eventlet",
        "requests",
        "six",
        "hypothesis",
        "psutil",
        "docker >= 3.5, < 3.6",
        "kubernetes == 10.0.0",
    ],
    zip_safe=False,
)



"""CSP config"""

from common.app_settings import domain, MODULE_NAME

CSP_CONFIG = {
    "CSP_DEFAULT_SRC": ("'self'",),
    "CSP_IMG_SRC": (
        f"{domain()}/static/",
        "https://p.typekit.net/",
    ),
    "CSP_FONT_SRC": ("https://use.typekit.net/",),
    "CSP_SCRIPT_SRC": (
        "https://use.typekit.net/mrl4ieu.js",
        "https://pyodide-cdn2.iodide.io/v0.15.0/full/",
    ),
    "CSP_CONNECT_SRC": (
        "https://pyodide-cdn2.iodide.io/v0.15.0/full/",
        f"wss://{MODULE_NAME}-aimmo.codeforlife.education/",
        f"https://{MODULE_NAME}-aimmo.codeforlife.education/",
    ),
}

if MODULE_NAME == "local":
    CSP_CONFIG["CSP_CONNECT_SRC"] += (
        "ws://localhost:*/",
        "ws://*:*/socket.io/",
        "http://*:*/socket.io/",
    )



# -*- coding: utf-8 -*-
import re
import sys

from setuptools import find_packages, setup

with open("aimmo/__init__.py", "r") as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

try:
    from semantic_release import setup_hook

    setup_hook(sys.argv)
except ImportError:
    pass

setup(
    name="aimmo",
    packages=find_packages(exclude=["*.tests", "*.tests.*"]),
    package_dir={"aimmo": "aimmo"},
    include_package_data=True,
    install_requires=[
        "django==2.2.26",
        "django-js-reverse==0.9.1",
        "djangorestframework==3.12.2",
        "eventlet==0.31.0",
        "requests==2.25.0",
        "hypothesis==5.41.3",
        "kubernetes==12.0.1",
        "cfl-common",
        "django-csp==3.7",
    ],
    tests_require=["docker >= 3.5, < 3.6", "kubernetes == 5.0.0", "PyYAML == 5.4"],
    version=version,
    zip_safe=False,
)



from simulation.interactables.conditions import avatar_on_cell
from simulation.interactables.effects import HealthEffect
from simulation.interactables.interactable import _Interactable


class HealthPickup(_Interactable):
    def __init__(self, cell):
        super(HealthPickup, self).__init__(cell)
        self.delete_after_effects_applied = True

        self.conditions.append(avatar_on_cell)
        self.effects.append(HealthEffect)

    def get_targets(self):
        return [self.cell.avatar]

    def __repr__(self):
        return "HealthPickup(Location={})".format(self.cell.location)

    def serialize(self):
        return {
            "type": "health",
            "location": {"x": self.cell.location.x, "y": self.cell.location.y},
        }



"""
Gets the worksheet we are on and all the configuration that goes with it
"""

from __future__ import annotations

import os
from dataclasses import dataclass
from typing import TYPE_CHECKING

from simulation.game_logic.map_updaters import PickupUpdater
from simulation.interactables.interactable import _Interactable
from simulation.interactables.pickups import (
    YellowOrbArtefact,
    ChestArtefact,
    KeyArtefact,
)
from .avatar_state_serializers import (
    worksheet1_avatar_state_serializer,
    worksheet2_avatar_state_serializer,
    worksheet3_avatar_state_serializer,
)

if TYPE_CHECKING:
    from typing import List, Callable, Dict, NewType
    from simulation.avatar.avatar_wrapper import AvatarWrapper
    from simulation.game_logic.map_updaters import _MapUpdater

    AvatarStateSerializer = NewType(
        "AvatarStateSerializer", Callable[[AvatarWrapper], Dict]
    )


@dataclass
class WorksheetData:
    worksheet_id: int
    era: str
    map_updaters: "List[_MapUpdater]"
    number_of_obstacle_textures: int
    avatar_state_serializer: AvatarStateSerializer = worksheet1_avatar_state_serializer


worksheets = {
    1: WorksheetData(
        worksheet_id=1,
        era="future",
        map_updaters=[PickupUpdater(pickup_types=[YellowOrbArtefact])],
        number_of_obstacle_textures=1,
        avatar_state_serializer=worksheet1_avatar_state_serializer,
    ),
    2: WorksheetData(
        worksheet_id=2,
        era="future",
        map_updaters=[PickupUpdater(pickup_types=[YellowOrbArtefact])],
        number_of_obstacle_textures=1,
        avatar_state_serializer=worksheet2_avatar_state_serializer,
    ),
    3: WorksheetData(
        worksheet_id=3,
        era="ancient",
        map_updaters=[PickupUpdater(pickup_types=[ChestArtefact, KeyArtefact])],
        number_of_obstacle_textures=1,
        avatar_state_serializer=worksheet3_avatar_state_serializer,
    ),
}


def get_worksheet_data() -> WorksheetData:
    worksheet_id = int(os.environ.get("worksheet_id", 1))
    return worksheets.get(worksheet_id, worksheets[1])
